<?php

declare(strict_types=1);

namespace Drupal\date_point\Data;

use Drupal\Core\StringTranslation\TranslatableMarkup as TM;

/**
 * Represents storage precision for date and time fields.
 */
enum Precision: string {

  case SECONDS = 'seconds';
  case MILLISECONDS = 'milliseconds';
  case MICROSECONDS = 'microseconds';

  /**
   * Returns numeric precision suitable for defining database schema.
   *
   * @return int<0, 6>
   */
  public function toInteger(): int {
    return match ($this) {
      self::SECONDS => 0,
      self::MILLISECONDS => 3,
      self::MICROSECONDS => 6,
    };
  }

  /**
   * Returns precision options.
   *
   * @return array<string, \Drupal\Core\StringTranslation\TranslatableMarkup>
   */
  public static function getOptions(): array {
    return [
      self::SECONDS->value => new TM('Seconds'),
      self::MILLISECONDS->value => new TM('Milliseconds'),
      self::MICROSECONDS->value => new TM('Microseconds'),
    ];
  }

  /**
   * {@selfdoc}
   */
  public static function fromGranularity(Granularity $granularity): self {
    return match ($granularity) {
      Granularity::MICROSECOND => self::MICROSECONDS,
      Granularity::MILLISECOND => self::MILLISECONDS,
      default => self::SECONDS,
    };
  }

}
