<?php

declare(strict_types=1);

namespace Drupal\date_point\Plugin\views\argument;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup as TM;
use Drupal\date_point\Data\Granularity;
use Drupal\date_point\Plugin\views\DateTimeTrait;
use Drupal\views\Attribute\ViewsArgument;
use Drupal\views\Plugin\views\argument\ArgumentPluginBase;

/**
 * Argument handler for datetime fields.
 *
 * @property \Drupal\views\Plugin\views\query\Sql $query
 *
 * @ingroup views_argument_handlers
 */
#[ViewsArgument('dp_date_time')]
final class DateTime extends ArgumentPluginBase {

  use DateTimeTrait;

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    $options = parent::defineOptions();
    $options['dp_granularity'] = ['default' => Granularity::SECOND->value];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    parent::buildOptionsForm($form, $form_state);
    $form['dp_granularity'] = [
      '#type' => 'select',
      '#title' => new TM('Granularity'),
      '#options' => Granularity::getOptions(),
      '#default_value' => $this->options['dp_granularity'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function query($group_by = FALSE): void {
    $this->ensureMyTable();
    $value = self::buildValue($this->argument);
    if (!$value) {
      // As there no validation for user input we just filter out all records
      // when the supplied date is not valid.
      $this->query->addWhereExpression(0, '1 = 2');
      return;
    }
    $this->query->addWhere(0, "$this->tableAlias.$this->realField", $value);
  }

}
